/*
 * network.h
 *
 *  Created on: 05.01.2013
 *      Author: AndreR
 */

#ifndef NETWORK_H_
#define NETWORK_H_

#include "stm32f10x.h"

#include "network/inet.h"
#include "commands.h"
#include "constants.h"

#include "FreeRTOS.h"
#include "semphr.h"
#include "queue.h"

#include "stm32_eth.h"

typedef int16_t(*UDPCallback)(uint8_t* pData, uint16_t dataLength);
typedef int16_t(*VisionCallback)(uint8_t* pData, uint16_t dataLength);

typedef struct _Network
{
	ETH_DMADESCTypeDef rxDescTab[NETWORK_NUM_DESCRIPTORS];
	ETH_DMADESCTypeDef txDescTab[NETWORK_NUM_DESCRIPTORS];
	ETH_DMADESCTypeDef* pRxDescToGet;
	ETH_DMADESCTypeDef* pTxDescToSet;
	uint8_t rxProcBuf[NETWORK_MAX_FRAME_SIZE];
	uint8_t txProcBuf[NETWORK_MAX_FRAME_SIZE];

	xQueueHandle eventQueue;
	volatile uint8_t macIRQ;

	xSemaphoreHandle hSendPacket;

	uint16_t ethernetIdent;

	struct	// own values used as source
	{
		MAC mac;
		IPv4Address ip;
		uint16_t port;
	} my;

	struct	// values used as unicast destination
	{
		MAC mac;
		IPv4Address ip;
		uint16_t port;
	} ucast;

	struct	// values used for multicast receive, send
	{
		MAC mac;			// 01-00-5E-05-17-02
		IPv4Address ip;		// 224.5.23.2
		uint16_t port;		// 10002
	} vision;

	struct
	{
		uint32_t respTime;
		uint8_t respCounter;
		uint32_t lastSendTime;
		uint8_t robustness;
	} igmp;

	UDPCallback pUDPCb;
	VisionCallback pVisionCb;
} Network;

extern Network network;

void NetworkInit();
void NetworkTask(void* params);
void NetworkChangeIP(IPv4Address ip);
void NetworkChangePort(uint16_t port);
int16_t NetworkSendPacket(PacketHeader* pHeader, uint8_t* pData);
void NetworkSetUDPCallback(UDPCallback cb);
void NetworkSetVisionCallback(VisionCallback cb);
int16_t NetworkSendIGMPv3MembershipReport(uint16_t numberOfGroupRecords, uint8_t* recordData, uint16_t recordDataLength);

#endif /* NETWORK_H_ */
