/*
 * spi1_2.c
 *
 *  Created on: 06.03.2011
 *      Author: AndreR
 */

#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "misc.h"

#include "spi3.h"
#include "constants.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

SPI spi3;

static xSemaphoreHandle hNewPacket;

void DMA2_Channel1_IRQHandler()
{
	SPIDmaRxInterrupt(&spi3);
}

void SPI3Callback()
{
	signed portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

	xSemaphoreGiveFromISR(hNewPacket, &xHigherPriorityTaskWoken);

	if(xHigherPriorityTaskWoken == pdTRUE)
		taskYIELD();
}

int16_t SPI3Init()
{
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI3, ENABLE);

	GPIO_PinRemapConfig(GPIO_Remap_SPI3, ENABLE);

	static uint8_t spi3TxData[SPI3_FIFO_SIZE];
	static uint8_t spi3RxData[SPI3_FIFO_SIZE];
	static uint8_t spi3DmaTxData[SPI3_DMA_SIZE];
	static uint8_t spi3DmaRxData[SPI3_DMA_SIZE];
	static uint8_t spi3CbData[SPI3_CB_SIZE];

	SPIData data3;
	data3.pDataTx = spi3TxData;
	data3.pDataRx = spi3RxData;
	data3.size = SPI3_FIFO_SIZE;
	data3.pDmaDataTx = spi3DmaTxData;
	data3.pDmaDataRx = spi3DmaRxData;
	data3.dmaSize = SPI3_DMA_SIZE;
	data3.pCbData = spi3CbData;
	data3.cbSize = SPI3_CB_SIZE;
	data3.dmaChRx = 1;
	data3.dmaChTx = 2;
	data3.pDma = DMA2;

	SPIInit(&spi3, SPI3, data3, &SPI3Callback);

	NVIC_InitTypeDef NVICInit;
	NVICInit.NVIC_IRQChannel = DMA2_Channel1_IRQn;
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_SPI3;
	NVICInit.NVIC_IRQChannelSubPriority = 0x00;
	NVIC_Init(&NVICInit);

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;	// SCK, MISO, MOSI
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &gpioInit);

	vSemaphoreCreateBinary(hNewPacket);

	return 0;
}

void SPI3Task(void* params)
{
	params = params;

	while(1)
	{
		xSemaphoreTake(hNewPacket, 10/portTICK_RATE_MS);

		SPINotifySlaveCallbacks(&spi3);
	}
}
