/*
 * wifi.h
 *
 *  Created on: 02.12.2012
 *      Author: AndreR, FriederB
 */

#ifndef WIFI_H_
#define WIFI_H_

#include "stm32f10x.h"
#include "hal/nrf24.h"
#include "network.h"
#include "util/fifo_packet_synced.h"
#include "constants.h"

#include "FreeRTOS.h"
#include "queue.h"
#include "semphr.h"

#define WIFI_MAX_ROBOTS NRF24_MAX_BOT_ID+1

typedef struct _WifiRobot
{
	uint8_t state;
	uint32_t lastExecTime;
	uint32_t noLinkCounter;
	uint8_t id;
	uint32_t lastVisionTime;
} WifiRobot;

typedef struct _WifiGlobal
{
	NRF24 nrf24;

	WifiRobot bots[WIFI_MAX_ROBOTS];

	uint32_t runtime;	// in [se-4]

	uint8_t scan;

	uint8_t sendBuf[WIFI_SEND_BUFFER_SIZE];

	uint32_t visionPauseTime;	// 50kHz, HFTicks, 20us

	FifoPacketSynced txFifo;
	xSemaphoreHandle hTxMutex;
}WifiGlobal;

extern WifiGlobal wifi;

void WifiInit();
void WifiTask(void* params);
void WifiChangeChannel(uint8_t channel);
void WifiChangeSpeed(uint8_t speed);
void WifiChangeMaxRt(uint8_t maxRt);
int16_t WifiChangeAckDelay(uint16_t delayUs);
void WifiSetVisionRate(uint16_t rateHz);
void WifiSendPose(uint8_t botId, float x, float y, float orient);

#endif /* WIFI_H_ */
