/*
 * init.c
 *
 *  Created on: 17.05.2011
 *      Author: AndreR
 */

#include "init.h"

#include "stm32f4xx_iwdg.h"

#include "FreeRTOS.h"
#include "task.h"

#include "cli.h"
#include "network.h"
#include "spi1_2_3.h"
#include "util/console.h"
#include "intercom1.h"
#include "intercom_constants.h"
#include "power.h"
#include "util/crc.h"
#include "config.h"
#include "bootloader.h"

#include "hal/buzzer.h"
#include "hal/exti.h"
#include "hal/usart1.h"
#include "hal/usart2.h"
#include "hal/hw_sync1.h"


void InitTask(void* params)
{
	params = params;

	PowerPreInit();
	PowerEnable5VSupply(1);

	// Configure and start Independent watchdog
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
	IWDG_SetPrescaler(IWDG_Prescaler_8);
	IWDG_SetReload(2500);	// approx. 500ms
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);

	// Initialize shared resources
	// --- HAL ---
	ConfigInit();
	BuzzerInit();
	ExtiInit();
	USART1Init();
	USART2Init();
	CRCInit();

	// --- GATEKEEPERS ---
	SPI1_2_3Init();

	// --- High-Level Systems ---
	Intercom1Init();
	CLIInit();
	PowerInit();
	NetworkInit();
	BootloaderInit();

	// -- CLI
	ConsolePrint("\f--- Bootloader (MAIN) ---\r\n");

	xTaskCreate(CLITask, (const signed char* const)"CLI", 1536, NULL, 1, NULL);
	ConsolePrint("CLI: \e[0;32mOK\e[0m\r\n");

	ConsolePrint("Waiting for co-processors...");
	HWSyncBarrier(&hwSync1);
	ConsolePrint("Ready\r\n");

	xTaskCreate(USARTTask, (const signed char* const)"USART2", 1024, &usart2, 4, NULL);
	vTaskDelay(100/portTICK_RATE_MS);	// give USART some time to reset RX
	xTaskCreate(Intercom1Task, (const signed char* const)"IC", 1024, NULL, 1, NULL);

	// -- SPI 1, 2, 3
	xTaskCreate(SPI1_2_3Task, (const signed char* const)"SPI", 256, NULL, 3, NULL);

	ConsolePrint("SPI 1 + 2 + 3: \e[0;32mOK\e[0m\r\n");

	// Create Tasks, MAX PRIORITY: 4
	xTaskCreate(NRF24RxTask, (const signed char* const)"NRF24RX", 1536, &network.nrf24, 3, NULL);

	xTaskCreate(PowerDummyTask, (const signed char* const)"Power", 512, NULL, 1, NULL);

	xTaskCreate(BootloaderTask, (const signed char* const)"Bootloader", 512, NULL, 2, NULL);

	ConsolePrint("%d Tasks started\r\n", uxTaskGetNumberOfTasks());
	ConsolePrint("HEAP usage: %d/%d Byte\r\n", configTOTAL_HEAP_SIZE-xPortGetFreeHeapSize(), configTOTAL_HEAP_SIZE);
	ConsolePrint("Initialization done.\r\n");

	vTaskSuspend(NULL);	// Disable this task
}
