#include "stm32f4xx.h"
#include "misc.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_exti.h"
#include "stm32f4xx_gpio.h"

#include "core_cm4.h"

#include "FreeRTOS.h"
#include "task.h"

#include "hal/led.h"
#include "hal/hw_sync1.h"
#include "hal/rtos_hf_timer.h"
#include "init.h"
#include "util/boot.h"

#include <string.h>

void NVIC_Configuration();

int main()
{
	if(!BootIsBootloaderSelected())
		BootJumpToApplication();

	BootSetBootloaderSelected(0);

	// Enable important clocks
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_DMA1 | RCC_AHB1Periph_DMA2, ENABLE);
	RCC_AHB1PeriphClockCmd(	RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB | RCC_AHB1Periph_GPIOC |
							RCC_AHB1Periph_GPIOD | RCC_AHB1Periph_GPIOE | RCC_AHB1Periph_GPIOF |
							RCC_AHB1Periph_GPIOG, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

	SCB->SHCSR |= SCB_SHCSR_USGFAULTENA_Msk | SCB_SHCSR_BUSFAULTENA_Msk | SCB_SHCSR_MEMFAULTENA_Msk;

	NVIC_Configuration();

	// Init sync mechanism for timed distributed startup
	HWSync1Init();

	// init LEDs to signal FreeRTOS startup failures
	LEDInit();

	LEDSet(LED_MAIN_GREEN | LED_MAIN_RED);

	RTOSHFTimerInit();

	xTaskCreate(InitTask, (const signed char* const)"Init", 256, NULL, 4, NULL);

	vTaskStartScheduler();

	return 0;
}

void NVIC_Configuration()
{
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x00);

	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);
}
