/*
 * config.c
 *
 *  Created on: 28.04.2013
 *      Author: AndreR
 */

#include "config.h"

#include "stm32f4xx_flash.h"
#include "util/nrf24.h"
#include "errors.h"

#include <string.h>

#define EEPROM_MAGIC_BYTE 0x5E

Config config;
const Config* pEeConfig = (Config*)0x08004000;

void ConfigInit()
{
	memcpy(&config, pEeConfig, sizeof(Config));

	if(config.magic != EEPROM_MAGIC_BYTE)
	{
		ConfigReset();

		ConfigSave();
	}
}

int16_t ConfigSave()
{
	FLASH_Unlock();

	// Clear pending flags (if any)
	FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_OPERR | FLASH_FLAG_WRPERR |	FLASH_FLAG_PGAERR | FLASH_FLAG_PGPERR|FLASH_FLAG_PGSERR);

	if(FLASH_EraseSector(FLASH_Sector_1, VoltageRange_3) != FLASH_COMPLETE)
	{
		return ERROR_CONFIG_FLASH_ERROR;
	}

	uint32_t eeAddr = (uint32_t)pEeConfig;
	uint32_t eeEndAddr = eeAddr + sizeof(Config);
	uint32_t* pRamConfig = (uint32_t*)&config;

	while(eeAddr < eeEndAddr)
	{
		if(FLASH_ProgramWord(eeAddr, *pRamConfig) != FLASH_COMPLETE)
		{
			return ERROR_CONFIG_FLASH_ERROR;
		}

		eeAddr += 4;
		pRamConfig++;
	}

	FLASH_Lock();

	return 0;
}

void ConfigReset()
{
	memset(&config, 0, sizeof(Config));

	config.magic = EEPROM_MAGIC_BYTE;
	config.botId = 14;
	config.channel = 100;
	config.speed = NRF24_SPEED_1M;
	config.gyroScale = 1.0f;
}

