/*
 * arm_mat_util_f32.c
 *
 *  Created on: 18.03.2013
 *      Author: AndreR
 */

#include "arm_math.h"

#include <string.h>

void arm_mat_zero_f32(arm_matrix_instance_f32* pMat)
{
	memset(pMat->pData, 0, sizeof(float)*pMat->numCols*pMat->numRows);
}

void arm_mat_fill_f32(arm_matrix_instance_f32* pMat, float value)
{
	for(uint32_t i = 0; i < pMat->numCols*pMat->numRows; i++)
	{
		pMat->pData[i] = value;
	}
}

arm_status arm_mat_identy_f32(arm_matrix_instance_f32* pMat)
{
#ifdef ARM_MATH_MATRIX_CHECK
	if(pMat->numCols != pMat->numRows)
		return ARM_MAT_SIZE_MISMATCH;
#endif /*      #ifdef ARM_MATH_MATRIX_CHECK    */

	arm_mat_zero_f32(pMat);

	for(uint16_t i = 0; i < pMat->numCols; i++)
		pMat->pData[i*pMat->numCols+i] = 1.0f;

	return ARM_MATH_SUCCESS;
}

/**
 * Danger! Assumes that pData storage is large enough.
 */
void arm_mat_copy_f32(arm_matrix_instance_f32* pSrc, arm_matrix_instance_f32* pDst)
{
	pDst->numCols = pSrc->numCols;
	pDst->numRows = pSrc->numRows;
	memcpy(pDst->pData, pSrc->pData, sizeof(float)*pSrc->numCols*pSrc->numRows);
}

void arm_mat_clip_f32(arm_matrix_instance_f32* pMat, float min, float max)
{
	for(uint32_t i = 0; i < pMat->numCols*pMat->numRows; i++)
	{
		if(pMat->pData[i] > max)
			pMat->pData[i] = max;
		if(pMat->pData[i] < min)
			pMat->pData[i] = min;
	}
}
