/*
 * console.h
 *
 *  Created on: 24.05.2011
 *      Author: AndreR
 */

#ifndef __CONSOLE_H__
#define __CONSOLE_H__

#include "stm32f4xx.h"

#include "util/fifo_packet_synced.h"

#include "FreeRTOS.h"
#include "semphr.h"

typedef void(*NotifyTxReady)(void*);

typedef struct _ConsoleData
{
	uint8_t* pDataTx;
	uint16_t txSize;

	uint8_t* pDataRx;
	uint16_t rxSize;

	uint8_t* pScanBuf;
	uint16_t scanSize;

	uint8_t* pPrintBuf;
	uint16_t printSize;

	NotifyTxReady txCallback;
	void* pUserData;
} ConsoleData;

typedef struct _Console
{
	ConsoleData data;

	FifoPacketSynced txFifo;
	FifoPacketSynced rxFifo;

	uint8_t cmdReady;

	xSemaphoreHandle hTxMutex;
	xSemaphoreHandle hRxMutex;
	xSemaphoreHandle hScanMutex;
	xSemaphoreHandle hPrintMutex;
} Console;

extern Console console;

void		ConsoleInit(ConsoleData data);
uint16_t	ConsoleNumCmdsReady();
uint8_t		ConsoleIsCmdReady();
int16_t		ConsoleCmdProcessed();
int16_t		ConsoleCmpCmd(const char* cmd);
int32_t		ConsoleScanCmd(const char* fmt, ...);
int32_t		ConsolePrint(const char* fmt, ...);
char*		ConsoleGetPrintBuf();
int16_t		ConsoleSendPrintBuf(); // this function assumes that the print buf is locked by user!
void		ConsoleLockPrintBuf();
void		ConsoleUnlockPrintBuf();

#endif
