/*
 * dma_gen.h
 *
 *  Created on: 13.11.2012
 *      Author: AndreR
 */

#ifndef DMA_GEN_H_
#define DMA_GEN_H_

#include "stm32f4xx.h"
#include "stm32f4xx_dma.h"

/**
 * pDMA: DMA1 or DMA2
 * stream: 0 - 7
 */
static __inline DMA_Stream_TypeDef* DMAGenerateStreamTypeDef(DMA_TypeDef* pDMA, uint32_t stream)
{
	return (DMA_Stream_TypeDef*)( ((uint32_t*)pDMA) +  6*stream + 4);
}

static __inline volatile uint32_t* DMAGenerateIFCR(DMA_TypeDef* pDMA, uint32_t stream)
{
	if(stream < 4)
	{
		return &pDMA->LIFCR;
	}
	else
	{
		return &pDMA->HIFCR;
	}
}

static __inline volatile uint32_t* DMAGenerateISR(DMA_TypeDef* pDMA, uint32_t stream)
{
	if(stream < 4)
	{
		return &pDMA->LISR;
	}
	else
	{
		return &pDMA->HISR;
	}
}

static __inline uint32_t DMAGenerateMaskAll(uint32_t stream)
{
	if(stream >= 4)
	{
		stream -= 4;
	}

	return (0x0000003D << (stream*6 + (stream > 1 ? 4 : 0)));
}

static __inline uint32_t DMAGenerateMaskTCIF(uint32_t stream)
{
	if(stream >= 4)
	{
		stream -= 4;
	}

	return (0x00000020 << (stream*6 + (stream > 1 ? 4 : 0)));
}

static __inline uint32_t DMAGenerateMaskTEIF(uint32_t stream)
{
	if(stream >= 4)
	{
		stream -= 4;
	}

	return (0x00000008 << (stream*6 + (stream > 1 ? 4 : 0)));
}


#endif /* DMA_GEN_H_ */
