/*
 * intercom.h
 *
 *  Created on: 10.11.2012
 *      Author: AndreR
 */

#ifndef INTERCOM_H_
#define INTERCOM_H_

#include "stm32f4xx.h"

#include "usart.h"

#include "FreeRTOS.h"
#include "semphr.h"

#define INTERCOM_MAX_CLIENTS 10

#define INTERCOM_HEADER_SIZE 3

typedef union __attribute__ ((packed)) _IntercomHeader
{
	struct
	{
		uint8_t address;
		uint8_t section;
		uint8_t command;
	};

	uint8_t _data[INTERCOM_HEADER_SIZE];
} IntercomHeader;

typedef void(*IntercomCallback)(IntercomHeader*, uint16_t, uint8_t*, void*);	//header, dataLength, data, userData

typedef struct _IntercomClient
{
	uint8_t section;
	IntercomCallback rxCallback;
	void* pUserData;
} IntercomClient;

typedef struct _Intercom
{
	USART* pUSART;

	IntercomClient clients[INTERCOM_MAX_CLIENTS];
	uint8_t numClientsUsed;

	xSemaphoreHandle hRxDone;

	uint8_t* pProcBuf;
	uint16_t procLength;

	uint8_t ownAddress;
} Intercom;

void IntercomInit(Intercom* pIntercom, USART* pUSART, uint8_t* pProcBuf, uint16_t procLength, uint8_t ownAddr);
int16_t IntercomSend(Intercom* pIntercom, IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData);
int16_t IntercomAddClient(Intercom* pIntercom, IntercomClient client);
int16_t IntercomProcess(Intercom* pIntercom, int32_t blockTime);

#endif /* INTERCOM_H_ */
