/*
 * sccb.c
 *
 *  Created on: 03.12.2012
 *      Author: AndreR
 */

#include "sccb.h"

#include "errors.h"

//////////////////////////////////////////////////
// SCCB testing
//////////////////////////////////////////////////

void SCCBInit(SCCB* pSCCB, GPIO_TypeDef* pPort, uint32_t clkPin, uint32_t dataPin, uint8_t address)
{
	pSCCB->pPort = pPort;
	pSCCB->clkPin = clkPin;
	pSCCB->dataPin = dataPin;
	pSCCB->address = address;

	pPort->BSRRL = clkPin | dataPin;

	// Configure SCCB GPIOs
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Pin = clkPin | dataPin;
	gpioInit.GPIO_Mode = GPIO_Mode_OUT;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(pPort, &gpioInit);
}

static void setDataOutput(SCCB* pSCCB)
{
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Pin = pSCCB->dataPin;
	gpioInit.GPIO_Mode = GPIO_Mode_OUT;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(pSCCB->pPort, &gpioInit);
}

static void setDataInput(SCCB* pSCCB)
{
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Pin = pSCCB->dataPin;
	gpioInit.GPIO_Mode = GPIO_Mode_IN;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(pSCCB->pPort, &gpioInit);
}

static inline void dataLow(SCCB* pSCCB)
{
	pSCCB->pPort->BSRRH = pSCCB->dataPin;
}

static inline void dataHigh(SCCB* pSCCB)
{
	pSCCB->pPort->BSRRL = pSCCB->dataPin;
}

static inline void clkLow(SCCB* pSCCB)
{
	pSCCB->pPort->BSRRH = pSCCB->clkPin;
}

static inline void clkHigh(SCCB* pSCCB)
{
	pSCCB->pPort->BSRRL = pSCCB->clkPin;
}

static inline uint32_t dataState(SCCB* pSCCB)
{
	return pSCCB->pPort->IDR & pSCCB->dataPin;
}


static void delay_us(uint32_t time)
{
	while(time--)
	{
		asm volatile ("nop");
	}
}

static void startSCCB(SCCB* pSCCB)
{
	dataHigh(pSCCB);
	delay_us(500);

	clkHigh(pSCCB);
	delay_us(500);

	dataLow(pSCCB);
	delay_us(500);

	clkLow(pSCCB);
	delay_us(500);
}

static void stopSCCB(SCCB* pSCCB)
{
	dataLow(pSCCB);
	delay_us(500);

	clkHigh(pSCCB);
	delay_us(500);

	dataHigh(pSCCB);
	delay_us(500);
}

static void noAck(SCCB* pSCCB)
{

	dataHigh(pSCCB);
	delay_us(500);

	clkHigh(pSCCB);
	delay_us(500);

	clkLow(pSCCB);
	delay_us(500);

	dataLow(pSCCB);
	delay_us(500);
}

static int16_t writeByte(SCCB* pSCCB, uint8_t m_data)
{
	uint8_t j;
	int16_t result = 0;

	for (j = 0; j < 8; j++)
	{
		if ((m_data << j) & 0x80)
		{
			dataHigh(pSCCB);
		}
		else
		{
			dataLow(pSCCB);
		}

		delay_us(500);
		clkHigh(pSCCB);
		delay_us(500);
		clkLow(pSCCB);
		delay_us(500);
	}

	delay_us(100);

	setDataInput(pSCCB);
	delay_us(500);

	clkHigh(pSCCB);
	delay_us(100);

	if (dataState(pSCCB))
	{
		result = ERROR_SCCB_WRITE_NACK;
	}

	clkLow(pSCCB);
	delay_us(500);

	setDataOutput(pSCCB);

	return result;
}

static uint8_t readByte(SCCB* pSCCB)
{
	uint8_t read, j;
	read = 0x00;

	setDataInput(pSCCB);
	delay_us(500);

	for (j = 8; j > 0; j--)
	{
		delay_us(500);

		clkHigh(pSCCB);
		delay_us(500);

		read = read << 1;
		if (dataState(pSCCB))
		{
			read = read + 1;
		}

		clkLow(pSCCB);
		delay_us(500);
	}

	setDataOutput(pSCCB);

	return (read);
}

int16_t SCCBWriteReg(SCCB* pSCCB, uint8_t regID, uint8_t regDat)
{
	startSCCB(pSCCB);
	if (writeByte(pSCCB, pSCCB->address) == ERROR_SCCB_WRITE_NACK)
	{
		stopSCCB(pSCCB);
		return ERROR_SCCB_WRITE_NACK;
	}
	delay_us(100);
	if (writeByte(pSCCB, regID) == ERROR_SCCB_WRITE_NACK)
	{
		stopSCCB(pSCCB);
		return ERROR_SCCB_WRITE_NACK;
	}
	delay_us(100);
	if (writeByte(pSCCB, regDat) == ERROR_SCCB_WRITE_NACK)
	{
		stopSCCB(pSCCB);
		return ERROR_SCCB_WRITE_NACK;
	}
	stopSCCB(pSCCB);

	return 0;
}

int16_t SCCBReadReg(SCCB* pSCCB, uint8_t regID, uint8_t *regDat)
{
	startSCCB(pSCCB);
	if (writeByte(pSCCB, pSCCB->address) == ERROR_SCCB_WRITE_NACK)
	{
		stopSCCB(pSCCB);
		return ERROR_SCCB_WRITE_NACK;
	}
	delay_us(100);
	if (writeByte(pSCCB, regID) == ERROR_SCCB_WRITE_NACK)
	{
		stopSCCB(pSCCB);
		return ERROR_SCCB_WRITE_NACK;
	}
	stopSCCB(pSCCB);

	delay_us(100);

	startSCCB(pSCCB);
	if (writeByte(pSCCB, pSCCB->address+1) == ERROR_SCCB_WRITE_NACK)
	{
		stopSCCB(pSCCB);
		return ERROR_SCCB_WRITE_NACK;
	}

	delay_us(100);

	*regDat = readByte(pSCCB);

	noAck(pSCCB);
	stopSCCB(pSCCB);

	return 0;
}
