/*
 * spi.h
 *
 *  Created on: 06.03.2011
 *      Author: AndreR
 */

#ifndef __SPI_H__
#define __SPI_H__

#include "stm32f4xx.h"
#include "stm32f4xx_spi.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_dma.h"

#include "fifo_packet_synced.h"

#include "FreeRTOS.h"
#include "semphr.h"

#define SPI_HEADER_SIZE 4
#define SPI_MAX_SLAVES 10

typedef union __attribute__ ((packed)) _SPIHeader
{
	struct
	{
		uint16_t length;
		uint8_t slaveId;
		uint8_t cmd;
	};

	uint8_t _data[SPI_HEADER_SIZE];
} SPIHeader;

typedef struct _SPIData
{
	uint8_t* pDataTx;
	uint8_t* pDataRx;
	uint16_t size;

	uint8_t* pDmaDataTx;
	uint8_t* pDmaDataRx;
	uint16_t dmaSize;

	uint8_t* pCbData;
	uint16_t cbSize;

	DMA_TypeDef* pDma;
	uint16_t dmaChTx;
	uint16_t dmaChRx;
} SPIData;

typedef void(*SPISlaveCallback)(SPIHeader*, uint8_t*, void*);	//header, data

typedef struct _SPISlave
{
	GPIO_TypeDef* pCSPort;
	uint16_t csPin;

	uint16_t prescaler;
	uint16_t cpol;
	uint16_t cpha;

	SPISlaveCallback callback;
	void* pUserData;
} SPISlave;

typedef void(*SPIPacketDoneCallback)();

typedef struct _SPI
{
	SPI_TypeDef* pRegister;
	uint32_t base;

	SPIData data;

	DMA_Stream_TypeDef* pDmaTx;
	DMA_Stream_TypeDef* pDmaRx;

	__IO uint32_t* pDMARxIFCR;
	__IO uint32_t* pDMATxIFCR;
	uint32_t IFCRRxMask;
	uint32_t IFCRTxMask;

	FifoPacketSynced txFifo;
	FifoPacketSynced rxFifo;

	SPISlave slaves[SPI_MAX_SLAVES];
	uint8_t numSlavesUsed;

	uint8_t exclusiveSlave;

	SPIHeader currentPacket;

	SPIPacketDoneCallback callback;

	volatile uint8_t busy;

	xSemaphoreHandle hTxFifoInUse;
	xSemaphoreHandle hExclusiveInUse;
} SPI;

void	SPIInit(SPI* pSPI, SPI_TypeDef* pRegister, SPIData data, SPIPacketDoneCallback doneCallback);
int16_t SPIPutPacket(SPI* pSPI, SPIHeader* pHeader, uint16_t dataLength, uint8_t* pData);
int16_t SPIAddSlave(SPI* pSPI, SPISlave slave, uint8_t* pSlaveId);
int16_t	SPISetExclusiveSlave(SPI* pSPI, uint8_t slaveId);
int16_t	SPIReleaseExclusiveSlave(SPI* pSPI);
int16_t SPINotifySlaveCallbacks(SPI* pSPI);
uint16_t SPINumPacketsToProcess(SPI* pSPI);
int16_t	SPIDmaRxInterrupt(SPI* pSPI);
uint8_t SPIIdle(SPI* pSPI);
int16_t SPISetExclusiveConfig(SPI* pSPI);
int16_t SPIGetPrescaler(SPI* pSPI, uint8_t slaveId, uint16_t* pPrescaler);
int16_t SPISetPrescaler(SPI* pSPI, uint8_t slaveId, uint16_t prescaler);

#endif
