/*
 * usart.h
 *
 *  Created on: 04.11.2012
 *      Author: AndreR
 */

#ifndef USART_H_
#define USART_H_

#include "stm32f4xx.h"
#include "stm32f4xx_usart.h"
#include "stm32f4xx_dma.h"
#include "stm32f4xx_gpio.h"

#include "util/fifo_packet_synced.h"

#include "FreeRTOS.h"
#include "semphr.h"

typedef struct _USARTData
{
	uint8_t* pDataTx;
	uint8_t* pDataRx;
	uint16_t size;

	uint8_t* pDmaDataTx;
	uint8_t* pDmaDataRx;
	uint16_t dmaSize;

	uint32_t dmaChannel;
	uint32_t rxStream;
	uint32_t txStream;

	DMA_TypeDef* pDMA;

	uint8_t rxIRQn;
	uint8_t txIRQn;
	uint8_t usartIRQn;
	uint8_t IRQL;

	uint32_t baudrate;

	GPIO_TypeDef* pTxSyncPort;
	uint16_t txSyncPin;
	GPIO_TypeDef* pRxSyncPort;
	uint16_t rxSyncPin;
} USARTData;

typedef void(*USARTCallback)(void*);

typedef struct _USART
{
	USART_TypeDef* pRegister;

	USARTData data;

	FifoPacketSynced txFifo;
	FifoPacketSynced rxFifo; // protect access with task mutex!

	DMA_Stream_TypeDef* pRxStream;
	DMA_Stream_TypeDef* pTxStream;

	__IO uint32_t* pDMARxIFCR;
	__IO uint32_t* pDMATxIFCR;
	__IO uint32_t* pDMARxISR;
	__IO uint32_t* pDMATxISR;
	uint32_t IFCRRxMask;
	uint32_t IFCRTxMask;

	xSemaphoreHandle hTxInUse;
	xSemaphoreHandle hRxInUse;

	USARTCallback callback;
	void* pUserData;

	volatile uint8_t txState;

	xQueueHandle hEventQueue;

	struct
	{
		uint32_t rxInvalid;
	} stats;
} USART;

void	USARTInit(USART* pUSART, USART_TypeDef* pRegister, USARTData data);
void	USARTTask(void* params);
int16_t USARTSend(USART* pUSART, uint16_t length, uint8_t* pData);
int16_t USARTSendConcat(USART* pUSART, uint16_t length, uint8_t* pData, uint16_t length2, uint8_t* pData2);
int16_t USARTStartTransfer(USART* pUSART);

// IRQ functions
void 	USARTDMATx(USART* pUSART);
void 	USARTDMARx(USART* pUSART);
void 	USARTIRQ(USART* pUSART);
void	USARTTxSyncFalling(USART* pUSART);

#endif /* USART_PACKET_H_ */
