/*
 * buzzer.c
 *
 *  Created on: 23.12.2010
 *      Author: AndreR
 */

#include "buzzer.h"

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_tim.h"
#include "stm32f4xx_rcc.h"
#include "misc.h"

#include "util/bit_banding.h"

#include "constants.h"

void BuzzerInit()
{
	// Timer init, TIM11
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM11, ENABLE);

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_Pin = GPIO_Pin_9;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOB, &gpioInit);

	GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_TIM11);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 1098;	// 1000Hz
	timeBaseInit.TIM_Prescaler = 83;	// = 1Mhz
	timeBaseInit.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM11, &timeBaseInit);

	TIM_OCInitTypeDef ocInit;
	ocInit.TIM_OCMode = TIM_OCMode_PWM1;
	ocInit.TIM_OutputState = TIM_OutputState_Enable;
	ocInit.TIM_Pulse = 1043;
	ocInit.TIM_OCPolarity = TIM_OCPolarity_High;
	ocInit.TIM_OCIdleState = TIM_OCIdleState_Reset;

	TIM_OC1Init(TIM11, &ocInit);

	TIM_OC1PreloadConfig(TIM11, TIM_OCPreload_Enable);

	TIM_ARRPreloadConfig(TIM11, ENABLE);

	TIM_Cmd(TIM11, ENABLE);

	TIM11->CCER &= ~TIM_CCER_CC1E;
}

void BuzzerEnable(uint8_t en)
{
	if(en)
	{
		TIM11->CCER |= TIM_CCER_CC1E;
	}
	else
	{
		TIM11->CCER &= ~TIM_CCER_CC1E;
	}
}

void BuzzerSetLevel(uint32_t level)
{
	TIM11->CCR1 = (TIM11->ARR*level)/100;
}

void BuzzerSetFrequency(uint32_t freq)
{
	TIM11->ARR = 1000000/freq;
	TIM11->CCR1 = 500000/freq;
}
