/*
 * l3g.c
 *
 *  Created on: 14.03.2013
 *      Author: AndreR
 */

#include "l3g.h"

#include <string.h>

#define L3G_DIGITAL_TO_RAD 0.0010652644360317f	// 2000 (FSS) / 2^15 (int range) * PI / 180

#define L3G_SET_CTRL 		10
#define L3G_GET_OUTPUT		11

#define L3G_READ_BIT		0x80
#define L3G_MULTIPLE_RW_BIT	0x40
#define REG_CTRL_REG1		0x20
#define REG_OUT_X_L			0x28

L3G l3g;

static void l3gCallback(SPIHeader* pHeader, uint8_t* pData, void* pUserData)
{
	(void) pUserData;

	if(pHeader->cmd != L3G_GET_OUTPUT)
		return;

	int16_t x = *((int16_t*)(pData+1));
	int16_t y = *((int16_t*)(pData+3));
	int16_t z = *((int16_t*)(pData+5));

	l3g.vel[0] = x*L3G_DIGITAL_TO_RAD;
	l3g.vel[1] = y*L3G_DIGITAL_TO_RAD;
	l3g.vel[2] = z*L3G_DIGITAL_TO_RAD;

	if(l3g.cb)
		(*l3g.cb)();
}

int16_t L3GInit(SPI* pSPI)
{
	SPIHeader header;
	SPISlave slave;
	uint8_t buf[5];
	int16_t result;

	memset(&l3g, 0, sizeof(L3G));

	l3g.pSPI = pSPI;

	// GYROSCOPE INITIALIZATION //

	slave.pCSPort = GPIOD;
	slave.csPin = GPIO_Pin_11; // GYR
	slave.cpol = 0;
	slave.cpha = 0;
	slave.prescaler = SPI_BaudRatePrescaler_8;
	slave.callback = &l3gCallback;
	slave.pUserData = 0;
	result = SPIAddSlave(pSPI, slave, &l3g.slaveId);
	if(result)
		return result;

	header.slaveId = l3g.slaveId;
	header.cmd = L3G_SET_CTRL;
	header.length = 5;

	buf[0] = REG_CTRL_REG1 | L3G_MULTIPLE_RW_BIT; // CTRL1 Register + multiple write bit
	buf[1] = 0xBF; // CTRL1: 400Hz, Cut-off 110Hz, X,Y,Z on
	buf[2] = 0x09; // CTRL2: High Pass 0.05Hz
	buf[3] = 0x00; // CTRL3: Data Ready interrupt
	buf[4] = 0x20; // CTRL4: FSS@2000dps

	result = SPIPutPacket(pSPI, &header, 5, buf);
	if(result)
		return result;

	return 0;
}

int16_t L3GUpdate()
{
	SPIHeader header;
	uint8_t buf[1];

	header.slaveId = l3g.slaveId;
	header.cmd = L3G_GET_OUTPUT;
	header.length = 7;
	buf[0] = REG_OUT_X_L | L3G_READ_BIT | L3G_MULTIPLE_RW_BIT; // OUT_X_L Register + read bit + multiple read bit

	int16_t result = SPIPutPacket(l3g.pSPI, &header, 1, buf);

	return result;
}

void L3GSetCallback(L3GCallback cb)
{
	l3g.cb = cb;
}
