/*
 * led.c
 *
 *  Created on: 30.10.2010
 *      Author: AndreR
 */

#include "led.h"

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"

#define LED0 (1 << 4)
#define LED1 (1 << 2)

typedef struct _LEDGlobal
{
	uint8_t state;		// Bit 0-7 I2C LEDs, Bit 8-9 primary LEDs
} LEDGlobal;

static LEDGlobal led;

void LEDInit()
{
	led.state = 0;

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Pin = LED0 | LED1;
	gpioInit.GPIO_Mode = GPIO_Mode_OUT;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOA, &gpioInit);
}

void LEDSet(uint8_t state)
{
	led.state = state;

	if(led.state & LED_MAIN_GREEN)
		GPIOA->BSRRH = LED0;
	else
		GPIOA->BSRRL = LED0;

	if(led.state & LED_MAIN_RED)
		GPIOA->BSRRH = LED1;
	else
		GPIOA->BSRRL = LED1;
}

void LEDMustOn(uint8_t on)
{
	LEDSet(led.state | on);
}

void LEDMustOff(uint8_t off)
{
	LEDSet(led.state & ~(off));
}

void LEDToggle(uint8_t toggle)
{
	LEDSet(led.state ^ toggle);
}

uint8_t LEDGetState()
{
	return led.state;
}
