/*
 * lis3.c
 *
 *  Created on: 14.03.2013
 *      Author: AndreR
 */

#include "lis3.h"

#include <string.h>

#define LIS3_DITIGAL_TO_MS2 0.00059855f	// g * 2 (FSS) / 2^15 (int range)

#define LIS3_SET_CTRL 		02
#define LIS3_GET_OUTPUT		03

#define LIS3_READ_BIT			0x80
#define LIS3_MULTIPLE_RW_BIT	0x40
#define REG_CTRL_REG1			0x20
#define REG_OUT_X_L				0x28

LIS3 lis3;

static void lis3Callback(SPIHeader* pHeader, uint8_t* pData, void* pUserData)
{
	(void) pUserData;

	if(pHeader->cmd != LIS3_GET_OUTPUT)
		return;

	int16_t x = *((int16_t*)(pData+1));
	int16_t y = *((int16_t*)(pData+3));
	int16_t z = *((int16_t*)(pData+5));

	lis3.acc[0] = x*LIS3_DITIGAL_TO_MS2;
	lis3.acc[1] = y*LIS3_DITIGAL_TO_MS2;
	lis3.acc[2] = z*LIS3_DITIGAL_TO_MS2;

	if(lis3.cb)
		(*lis3.cb)();
}

int16_t LIS3Init(SPI* pSPI)
{
	SPIHeader header;
	SPISlave slave;
	uint8_t buf[5];
	int16_t result;

	memset(&lis3, 0, sizeof(LIS3));

	lis3.pSPI = pSPI;

	// ACCELEROMETER INITIALIZATION //

	slave.pCSPort = GPIOD;
	slave.csPin = GPIO_Pin_10; // ACC
	slave.cpol = 0;
	slave.cpha = 0;
	slave.prescaler = SPI_BaudRatePrescaler_8;
	slave.callback = &lis3Callback;
	slave.pUserData = 0;
	result = SPIAddSlave(pSPI, slave, &lis3.slaveId);
	if(result)
		return result;

	header.slaveId = lis3.slaveId;
	header.cmd = LIS3_SET_CTRL;
	header.length = 5;

	buf[0] = REG_CTRL_REG1 | LIS3_MULTIPLE_RW_BIT; // CTRL1 Register + multiple write bit
	buf[1] = 0x77; // CTRL1: 400Hz, X,Y,Z on
	buf[2] = 0x38; // CTRL2: Enable High-Pass Filter, cut-off at 1Hz
	buf[3] = 0x00; // CTRL3: All interrupts disabled
	buf[4] = 0x88; // CTRL4: BDU + High Resolution output + FSS: 2G

	result = SPIPutPacket(pSPI, &header, 5, buf);
	if(result)
		return result;

	return 0;
}

int16_t LIS3Update()
{
	SPIHeader header;
	uint8_t buf[1];

	header.slaveId = lis3.slaveId;
	header.cmd = LIS3_GET_OUTPUT;
	header.length = 7;
	buf[0] = REG_OUT_X_L | LIS3_READ_BIT | LIS3_MULTIPLE_RW_BIT; // OUT_X_L Register + read bit + multiple read bit

	int16_t result = SPIPutPacket(lis3.pSPI, &header, 1, buf);

	return result;
}

void LIS3SetCallback(LIS3Callback cb)
{
	lis3.cb = cb;
}
