/*
 * kicker.h
 *
 *  Created on: 19.03.2011
 *      Author: AndreR
 */

#ifndef __KICKER_H__
#define __KICKER_H__

#include "stm32f4xx.h"

#include "util/atmega.h"
#include "atmega/kicker_prog/kicker_data.h"

#include "FreeRTOS.h"
#include "queue.h"
#include "semphr.h"

#include "commands.h"

#define KICKER_MODE_FORCE		0
#define KICKER_MODE_ARM			1
#define KICKER_MODE_DISARM		2
#define KICKER_MODE_DRIBBLER	3

typedef struct _Kicker
{
	KickerData data;
	uint8_t rxData[KICKER_DATA_SIZE+1];
	uint8_t txData[KICKER_DATA_SIZE+1];

	ATMega atmega;

	xQueueHandle hEventQueue;

	uint8_t state;
	struct
	{
		float fireDuration;
		uint8_t device;
	} armed;
	uint8_t doDischarge;
	uint32_t lastKick;

	KickerStatusV3 latestStatus;

	uint8_t logEnabled;

	uint8_t attached;
} Kicker;

extern Kicker kicker;

void KickerInit();
void KickerTask(void* params);

/**
 * Manually load the kicker.
 *
 * @param tOn ON time in ticks [1/12 se-6]
 * @param tOff OFF time in ticks [1/12 se-6]
 * @param duration Charge duration in [ms] (< 6553ms)
 *
 * @note tOn + tOff must be < 65536
 */
void KickerLoadManual(uint16_t tOn, uint16_t tOff, uint16_t duration);
void KickerLoadAuto(uint16_t maxCap);

/**
 * Fire the kicker!
 * @param duration Firing duration in [us] (max: 20ms)
 * @param device KICKER_DEVICE_STRAIGHT or KICKER_DEVICE_CHIP
 */
void KickerFire(float duration, uint8_t device);
void KickerArmIr(float fireDuration, uint8_t device);
void KickerArmDribbler(float fireDuration, uint8_t device);
void KickerDisarm();

void KickerDischarge();

/**
 * Set kicker voltage, current and ir voltage logging.
 * @param enable 0 or 1
 */
void KickerSetLogging(uint8_t enable);

#endif
