/*
 * gui.c
 *
 *  Created on: 06.01.2013
 *      Author: AndreR
 */

#include "gui.h"
#include "display.h"
#include "hal/tft.h"
#include "task.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "util/console.h"
#include "fonts.h"

typedef struct _ButtonCoordinates
{
	uint16_t upperleftX;
	uint16_t upperleftY;
	uint16_t lowerrightX;
	uint16_t lowerrightY;
	float number;
} ButtonCoordinates;

typedef struct _NumButtonCoordinates
{
	uint16_t upperleftX;
	uint16_t upperleftY;
	uint16_t lowerrightX;
	uint16_t lowerrightY;
	uint16_t fGColor;
	uint16_t bGColor;
	uint8_t number;
	uint8_t rotation;
} NumButtonCoordinates;

void GUIHandleNumButtonPressed(uint8_t type, uint8_t num);
uint16_t GUIDrawButton(uint16_t x1, uint16_t y1, uint8_t * str, uint32_t bGColor, uint32_t fGcolor, uint16_t width);

void GUIDrawTabContent(void);
void GUIDrawTabBar(void);
void GUIDrawTabPage(uint8_t page);
uint8_t GUIHandleNextButtonPressed( uint8_t page, uint8_t direction);

char numString[40];

static NumButtonCoordinates numBtnCoor[MAX_NUM_BTNS];
uint8_t numBtnCounter = 0;

uint8_t digitCounter = 0;
uint8_t hasDecimalPoint = 0;
uint8_t numPadActive = 0;
uint8_t useInput = 0;
signed char vZ = 1;
void* numToBeChangedPtr;

uint8_t i, q;
GUI gui;
uint8_t currPage = 0;
uint8_t currTabNum = 0;

static void resetNumString(void)
{
	digitCounter = 0;
	numString[0] = ' ';
	for (uint8_t ctr = 1; ctr < 40; ctr++)
		numString[ctr] = 0;
}

static void drawChar(PointU16 pos, uint8_t ch, uint32_t bGColor, uint32_t fGColor, uint8_t rotation)
{
	uint32_t i;
	int32_t j;

	if(rotation == 1 || rotation == 3)
		TFTSetPixelArea(RectU16PosSize(pos, SizeU16WH(12, 6)), rotation);
	else
		TFTSetPixelArea(RectU16PosSize(pos, SizeU16WH(6, 12)), rotation);

	// Write Image
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for (i = (ch - 0x20) * 12; i < (ch - 0x20 + 1) * 12; i++)
	{
		for (j = 7; j >= 2; j--)
		{
			if (fontSmall[i] & (1 << j))
				TFTWritePixel(fGColor); // bmp to C program says it produces RGB565 format already
			else
				TFTWritePixel(bGColor);
		}  // for
	} // for
}

static void drawString(PointU16 pos, const char * str, uint32_t bGColor, uint32_t fGColor, uint8_t rotation)
{
	int16_t i;

	switch(rotation)
	{
	case 0:
		for(i = 0; i < strlen(str); i++)
		{
			drawChar(pos, str[i], bGColor, fGColor, 0);
			pos.x += 6;
		}
		break;
	case 1:
		for(i = 0; i < strlen(str); i++)
		{
			drawChar(pos, str[i], bGColor, fGColor, 3);
			pos.y += 6;
		}
		break;
	/*
	case 2:
		for (i = strlen((const char *)str)-1; i >= 0; i--)
			GUIDrawChar(x1+i*8, y1, str[strlen((const char *)str)-1-i], bGColor, fGColor, 0);
		break;
	case 3:
		for (i = strlen((const char *)str)-1; i >= 0; i--)
			GUIDrawChar(x1, y1+i*8, str[strlen((const char *)str)-1-i], bGColor, fGColor, 0);
		break;
	*/
	}
}

static void drawBigChar(PointU16 pos, uint8_t ch, uint32_t bGColor, uint32_t fGColor, uint8_t rotation)
{
	uint32_t i;
	int32_t j;

	if(rotation == 1 || rotation == 3)
		TFTSetPixelArea(RectU16PosSize(pos, SizeU16WH(16, 12)), rotation);  //16, 12
	else
		TFTSetPixelArea(RectU16PosSize(pos, SizeU16WH(12, 16)), rotation);

	// Write Image
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(i = (ch - 0x20) * 32; i < (ch - 0x20 + 1) * 32; i++)
	{
		if(i%2)
		{
			for(j = 7; j >= 2; j--)
			{
				if(fontBig[i] & (1 << j))
					TFTWritePixel(fGColor);
				else
					TFTWritePixel(bGColor);
			}  // for
		}
		else
		{
			for(j = 5; j >= 0; j--)
			{
				if(fontBig[i] & (1 << j))
					TFTWritePixel(fGColor);
				else
					TFTWritePixel(bGColor);
			}
		}
	}
}

static void drawBigString(PointU16 pos, const char * str, uint32_t bGColor, uint32_t fGColor, uint8_t rotation)
{
	int16_t i;
	switch(rotation)
	{
		case 0:
			for(i = 0; i < strlen(str); i++)
			{
				drawBigChar(pos, str[i], bGColor, fGColor, 0);
				pos.x += 12;
			}
			break;
		case 1:
			for(i = 0; i < strlen(str); i++)
			{
				drawBigChar(pos, str[i], bGColor, fGColor, 1);
				pos.y -= 12;
			}
			break;
		case 3:
			for(i = 0; i < strlen(str); i++)
			{
				drawBigChar(pos, str[i], bGColor, fGColor, 3);
				pos.y += 12;
			}
			break;
		/*
		case 2:
			for (i = strlen((const char *)str)-1; i >= 0; i--)
				GUIDrawBigChar(x1+i*14, y1, str[strlen((const char *)str)-1-i], bGColor, fGColor, 0);
			break;
		case 3:
			for (i = strlen((const char *)str)-1; i >= 0; i--)
				GUIDrawBigChar(x1, y1+i*14, str[strlen((const char *)str)-1-i], bGColor, fGColor, 0);
			break;
		*/
	}
}

static void drawSevenSegNumber(PointU16 pos, uint8_t num, uint32_t bGColor, uint32_t fGColor, uint8_t rotation  )
{
 	uint32_t i;
 	int32_t j;

 	TFTSetPixelArea(RectU16PosSize(pos, SizeU16WH(50, 32)), rotation);

	// Write Image
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(i = num*200; i < (num+1)*200; i++)
	{
		for(j=7; j>=0; j--)
		{
			if(fontSevenSegNum[i]&(1<<j))
				TFTWritePixel(fGColor);  // bmp to C program says it produces RGB565 format already
			else
				TFTWritePixel(bGColor);
		}
	}
}

static void drawIcon(PointU16 pos, uint8_t* iconPtr, uint8_t rotation)
{
	uint16_t i;
	TFTSetPixelArea(RectU16PosSize(pos, SizeU16WH(32, 32)), rotation);

	// Write Image
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for (i = 0; i < 1024; i++)
	{
		*TFT_DATA_REG = iconPtr[i];  // bmp to C program says it produces RGB565 format already
	}
}

static void drawNumButton(PointU16 pos, uint8_t num, uint32_t bGColor, uint32_t fGColor, uint8_t rotation )
{
	uint32_t y, x;

	TFTSetPixelArea(RectU16PosSize(pos, SizeU16WH(54, 54)), 0);

	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(y = 0; y < 54; y++)
	{
		for(x = 0; x < 54; x++)
		{
			if(y == 0 || y==1 || y == 52 || y==53 || x==0 || x == 1 || x==52 || x == 53)
				TFTWritePixel(fGColor);
			else
				TFTWritePixel(bGColor);
		}
	}

	pos.x += 2;
	pos.y += 11;
	drawSevenSegNumber(pos, num, bGColor, fGColor, rotation );
}

static void drawNumPad(uint32_t bGColor, uint32_t fGColor, uint8_t type, uint8_t rotation)
{
	uint8_t num = 0;

	TFTFillAll(0);

	for(uint8_t i = 4; i > 0; i--)
	{
		for(uint8_t j = 0; j < 4; j++)
		{
			if(type == FLOAT || (type == POSITIVE_FLOAT && num != 11) || (type == INTEGER && num != 10) || (type == POSITIVE_INTEGER && num != 10 && num != 11)   )
				drawNumButton(PointU16XY((i-1)*56+3,  j*56+3), num, bGColor, fGColor, rotation   );
			num++;
		}
	}
}

static void addNumButton(uint16_t x1, uint16_t y1, uint8_t num, uint32_t bGColor, uint32_t fGColor, uint8_t rotation)
{
	if (numBtnCounter < MAX_NUM_BTNS )
	{
		numBtnCoor[numBtnCounter].upperleftX = x1;
		numBtnCoor[numBtnCounter].upperleftY = y1;
		numBtnCoor[numBtnCounter].lowerrightX = x1+53;
		numBtnCoor[numBtnCounter].lowerrightY = y1+53;
		numBtnCoor[numBtnCounter].fGColor = fGColor;
		numBtnCoor[numBtnCounter].bGColor = bGColor;
		numBtnCoor[numBtnCounter].number = num;
		numBtnCoor[numBtnCounter].rotation = rotation;
		numBtnCounter++;
		drawNumButton(PointU16XY(x1, y1),  num, bGColor, fGColor, rotation);
	}
}

static void addNumPad(uint32_t bGColor, uint32_t fGColor, uint8_t rotation)
{
	uint8_t i, j, num=0;
	for (i=4; i>0; i--)
	{
		for (j=0; j<4; j++)
		{
			numBtnCoor[num].upperleftX = (i-1)*56+3;
			numBtnCoor[num].upperleftY = j*56+3;
			numBtnCoor[num].lowerrightX = (i-1)*56+56;
			numBtnCoor[num].lowerrightY = j*56+56;
			numBtnCoor[num].bGColor = bGColor;
			numBtnCoor[num].fGColor = fGColor;
			numBtnCoor[num].number = num;
			numBtnCoor[num].rotation = rotation;
			num++;
		}
	}
}

static void resetNumButton(uint8_t btnNr)
{
	numBtnCoor[btnNr].upperleftX = 1337;  // somewhere beyond the display pixels
	numBtnCoor[btnNr].upperleftY = 1337;
	numBtnCoor[btnNr].lowerrightX = 1337;
	numBtnCoor[btnNr].lowerrightY = 1337;
	// numBtnCoor[btnNr].number = 133;
	// note that non-usable gap in the array arises when function is used
}

void GUIInit(void)
{
	resetNumString();

	gui.numberOfTabs = 0;

	GUIDrawTabBar();
	GUIDrawTabPage(currPage);
	addNumPad(0, 0xff00, 3);
}

void GUIDrawTabContent(void)
{
	TFTFillRect(RectU16Coordinates(0, 0, LCD_HPS-TAB_BAR_WIDTH, LCD_VPS), 0);

	for (i = 0; i <  CURR_TAB.numElements; i++)
	{
//		ConsolePrint("checking tab content...  \n\r\n");
//		//vTaskDelay(10/portTICK_RATE_MS);
//		ConsolePrint("Tab number: %hhu\r\n", gui.activeTab);
//		ConsolePrint("Element number: %hhu\r\n", i);
//		ConsolePrint("Background Color: %06X\r\n", CURR_ELEMENT.bGColor);
//		ConsolePrint("Foreground Color: %06X\r\n", CURR_ELEMENT.fGColor);
//		ConsolePrint("posX: %hu\r\n", CURR_ELEMENT.pos.x);
//		ConsolePrint("posY: %hu\r\n", CURR_ELEMENT.pos.y);
//		ConsolePrint("Type: %hhu\r\n", CURR_ELEMENT.type);

		switch(CURR_ELEMENT.type)
		{
			case ELEMENT_TYPE_TEXT:
			{
				drawString(CURR_ELEMENT.pos, CURR_ELEMENT.text.pText , CURR_ELEMENT.bGColor, CURR_ELEMENT.fGColor, 1 );
			}
			break;
			case ELEMENT_TYPE_BUTTON:
			{
				TFTDrawRect(RectU16PosSize(CURR_ELEMENT.pos, CURR_ELEMENT.button.size), CURR_ELEMENT.fGColor);
				drawBigString(PointU16Add(CURR_ELEMENT.pos, PointU16XY(1, 1)), CURR_ELEMENT.button.pText, CURR_ELEMENT.bGColor, CURR_ELEMENT.fGColor, 3);
			}
			break;
			case ELEMENT_TYPE_INPUT_INT:
			{
				sprintf(numString, "%i", *CURR_ELEMENT.inputFieldInt.pValue);
				TFTDrawRect(RectU16PosSize(CURR_ELEMENT.pos, SizeU16WH(16, CURR_ELEMENT.inputFieldInt.width)), CURR_ELEMENT.fGColor);
				drawBigString(PointU16Add(CURR_ELEMENT.pos, PointU16XY(1, 1)), numString, CURR_ELEMENT.fGColor, CURR_ELEMENT.bGColor, 3 );
			}
			break;
			case ELEMENT_TYPE_INPUT_FLOAT:
			{
				TFTDrawRect(RectU16PosSize(CURR_ELEMENT.pos, SizeU16WH(16, CURR_ELEMENT.inputFieldFloat.width)), CURR_ELEMENT.fGColor);
				drawBigString(PointU16Add(CURR_ELEMENT.pos, PointU16XY(1, 1)), numString, CURR_ELEMENT.fGColor, CURR_ELEMENT.bGColor, 3 );
			}
			break;
		}
	}
//	ConsolePrint("Tab Content Drawn\r\n");
}

uint8_t GUIAddTab(GUITab* pTab)
{
	if(gui.numberOfTabs == MAX_TAB_NUM)
		return 0;

	gui.pTabs[gui.numberOfTabs] = pTab;
	currPage = gui.numberOfTabs;
	gui.numberOfTabs++;
	GUIDrawTabPage(currPage);
	GUIDrawTabContent();
//	ConsolePrint("GUI has now %hhu tabs\r\n", gui.numberOfTabs);
//	vTaskDelay(10/portTICK_RATE_MS);
//	currTabNum++;  // ?
	return 1;
}

void GUIDrawTabBar(void)
{
	TFTFillAll(0);
//	GUIDrawString(70, 40, "test4", 0, 0xff00, 0);
//	GUIDrawBigString(70, 10, "test3", 0, 0xff00, 0);
//	GUIDrawBigString(0, 0, "test", 0, 0xff00, 3);
//	GUIDrawBigString(10, 180, "test5", 0, 0xff00, 1);
//	GUIDrawString(40, 10, "test2", 0, 0xff00, 1);
	//TFTFillPart(LCD_HPS-TAB_BAR_WIDTH+1, 0, LCD_HPS, LCD_VPS, 0); //superfluous
	for (uint8_t k = 0; k < 5; k++)
		TFTDrawRect(RectU16Coordinates(LCD_HPS-TAB_BAR_WIDTH+1, k*48, LCD_HPS, (k+1)*48-1), 0xff00 );
}

void GUIDrawTabPage(uint8_t page)
{
	ConsolePrint("Drawing tab page %hhu\r\n", page);
	vTaskDelay(10/portTICK_RATE_MS);
	uint8_t k = 1;
	for(uint8_t j = page*3; j < (page+1)*3; j++, k++)
	{
		if(j < gui.numberOfTabs)
		{
			if(j != gui.activeTab)
			{
				TFTFillRect(RectU16Coordinates(LCD_HPS-TAB_BAR_WIDTH+2, k*48+1, LCD_HPS-1, (k+1)*48-2), 0);
				drawBigString(PointU16XY(LCD_HPS-18, k*48+2), (*(gui.pTabs[j])).pTitle, 0, 0xff00, 3);
			}
			else
			{
				TFTFillRect(RectU16Coordinates(LCD_HPS-TAB_BAR_WIDTH+2, k*48+1, LCD_HPS-1, (k+1)*48-2), 0xff00);
				drawBigString(PointU16XY(LCD_HPS-18, k*48+2), (*(gui.pTabs[j])).pTitle , 0xff00, 0, 3);
			}
		}
		else
			TFTFillRect(RectU16Coordinates(LCD_HPS-TAB_BAR_WIDTH+2, k*48+1, LCD_HPS-1, (k+1)*48-2), 0);
	}
	ConsolePrint("tab page drawn\r\n");
}

uint8_t GUIHandleNextButtonPressed( uint8_t page, uint8_t direction)
{
	if (direction)
	{
		page++;
		if (page > gui.numberOfTabs/3)
			page = 0;
		ConsolePrint("clicked right button, new page: %hhu\r\n", page);
		vTaskDelay(10/portTICK_RATE_MS);
	}
	else
	{
		if (page == 0)
			page = gui.numberOfTabs/3;
		else
			page--;
		ConsolePrint("clicked left button, new page: %hhu\r\n", page);
		vTaskDelay(10/portTICK_RATE_MS);
	}
	return page;
}

void GUIHandler(void)
{
	if(display.touchPressed)
	{
		display.touchPressed = 0;
		vTaskDelay(50/portTICK_RATE_MS);	// debouncing

		if(display.touchX > LCD_HPS-TAB_BAR_WIDTH )
		{
			if(display.touchY < 48)
			{
				currPage = GUIHandleNextButtonPressed(currPage, 0);
				ConsolePrint("trying to draw tab page %hhd\r\n", currPage);
				GUIDrawTabPage(currPage);  // overpainting
			}
			else if(display.touchY >= 192)
			{
				currPage = GUIHandleNextButtonPressed(currPage, 1);
				ConsolePrint("trying to draw tab page %hhd\r\n", currPage);
				GUIDrawTabPage(currPage);  // overpainting
			}
			else
			{
				if(display.touchY/48+currPage*3-1 < gui.numberOfTabs)
					gui.activeTab = display.touchY/48+currPage*3-1;

				ConsolePrint("active tab is now: %hhd\r\n", gui.activeTab);
				ConsolePrint("trying to redraw tab page %hhd\r\n", currPage);
				GUIDrawTabPage(currPage);  // overpainting
				GUIDrawTabContent();
			}
		}
		else if(numPadActive)
		{
			//numPadWasActive = 1;
			for(q = 0; q < MAX_NUM_BTNS; q++)
			{
				// check whether touched area lies within a number button
				if (display.touchX > numBtnCoor[q].upperleftX && display.touchX < numBtnCoor[q].lowerrightX && display.touchY > numBtnCoor[q].upperleftY && display.touchY < numBtnCoor[q].lowerrightY )
				{
					GUIHandleNumButtonPressed(numPadActive, numBtnCoor[q].number);
					drawNumButton(PointU16XY(numBtnCoor[q].upperleftX, numBtnCoor[q].upperleftY), numBtnCoor[q].number, numBtnCoor[q].fGColor, numBtnCoor[q].bGColor, numBtnCoor[q].rotation );  // background and foreground colors are swapped to visualize the touch
					break;
				}
			}
			if(!numPadActive)
			{
				//numPadWasActive = 0;
				// redraw the former display content
				GUIDrawTabBar();
				GUIDrawTabPage(currPage);
				GUIDrawTabContent();
				if (useInput)
					ConsolePrint("number in input field has been changed\r\n");
				else
					ConsolePrint("number in input field has not been changed\r\n");

			}
		}// if (numPadActive)
		else
		{
			for(i = 0; i < CURR_TAB.numElements; i++)
			{
				if(CURR_ELEMENT.type == ELEMENT_TYPE_BUTTON  && IS_IN_AREA(display.touchX, display.touchY, CURR_ELEMENT.pos.x, CURR_ELEMENT.pos.y, CURR_ELEMENT.pos.x + CURR_ELEMENT.button.size.height-1, CURR_ELEMENT.pos.y + CURR_ELEMENT.button.size.width-1) )
				{
					(*CURR_ELEMENT.button.cb)(255);
				}
				else if(CURR_ELEMENT.type == ELEMENT_TYPE_INPUT_INT && IS_IN_AREA(display.touchX, display.touchY, CURR_ELEMENT.pos.x, CURR_ELEMENT.pos.y, CURR_ELEMENT.pos.x + 17, CURR_ELEMENT.pos.y + CURR_ELEMENT.inputFieldInt.width-1) )
				{
					numToBeChangedPtr = (void*)(CURR_ELEMENT.inputFieldInt.pValue);
					numPadActive = CURR_ELEMENT.type;
					drawNumPad(0, 0x00ff00, numPadActive, 3);
				}
				else if(CURR_ELEMENT.type == ELEMENT_TYPE_INPUT_POS_INT && IS_IN_AREA(display.touchX, display.touchY, CURR_ELEMENT.pos.x, CURR_ELEMENT.pos.y, CURR_ELEMENT.pos.x + 17, CURR_ELEMENT.pos.y + CURR_ELEMENT.inputFieldUInt.width-1) )
				{
					numToBeChangedPtr = (void*)(CURR_ELEMENT.inputFieldUInt.pValue);
					numPadActive = CURR_ELEMENT.type;
					drawNumPad(0, 0x00ff00, numPadActive, 3);
				}
				else if((CURR_ELEMENT.type == ELEMENT_TYPE_INPUT_FLOAT || CURR_ELEMENT.type == ELEMENT_TYPE_INPUT_POS_FLOAT ) && IS_IN_AREA(display.touchX, display.touchY, CURR_ELEMENT.pos.x, CURR_ELEMENT.pos.y, CURR_ELEMENT.pos.x + 17, CURR_ELEMENT.pos.y + CURR_ELEMENT.inputFieldUInt.width-1) )
				{
					numToBeChangedPtr = (void*)(CURR_ELEMENT.inputFieldFloat.pValue);
					numPadActive = CURR_ELEMENT.type;
					drawNumPad(0, 0x00ff00, numPadActive, 3);
				}
			}
		}

	}  // if(display.touchPressed)

	if(display.touchReleased)
	{
		display.touchReleased = 0;
		vTaskDelay(2/portTICK_RATE_MS);	// debouncing
		// redraw the button that has been released
		if(numPadActive == FLOAT || (numPadActive == POSITIVE_FLOAT && numBtnCoor[q].number != 11) || (numPadActive == INTEGER && numBtnCoor[q].number != 10) || (numPadActive == POSITIVE_INTEGER && numBtnCoor[q].number != 10 && numBtnCoor[q].number != 11))
			drawNumButton(PointU16XY(numBtnCoor[q].upperleftX, numBtnCoor[q].upperleftY), numBtnCoor[q].number, numBtnCoor[q].bGColor, numBtnCoor[q].fGColor, numBtnCoor[q].rotation );
	}
}

void GUIHandleNumButtonPressed(uint8_t type, uint8_t num)
{
	// 10: decimal point, 11: minus, 12: undo last cipher, 13: abort, 14: enter
	switch (num)
	{
	case 0:
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
	case 9:
		digitCounter ++;
		numString[digitCounter] = num + '0';
		drawChar(PointU16XY(NUM_PAD_INPUT_X_POS, 2+digitCounter*8), num+'0', 0, 0xff00, 3);
		break;
	case 10: // decimal point
		if (hasDecimalPoint == 0 && ( type == FLOAT || type == POSITIVE_FLOAT ) )
		{
			hasDecimalPoint = 1;
			digitCounter ++;
			drawChar(PointU16XY(NUM_PAD_INPUT_X_POS, 2+digitCounter*8), '.', 0, 0xff00, 3);
		}
		break;
	case 11: //minus
		if (type == FLOAT || type == INTEGER)
		{
			vZ *= -1;
			if (vZ < 0)
			{
				drawChar(PointU16XY(NUM_PAD_INPUT_X_POS, 2), '-', 0, 0xff00, 3);
				numString[0] = '-';
			}
			else
			{
				drawChar(PointU16XY(NUM_PAD_INPUT_X_POS, 2), ' ', 0, 0xff00, 3);
				numString[0] = ' ';
			}
		}
		break;
	case 12: // undo last cipher
		numString[digitCounter] = 0;
		drawChar(PointU16XY(NUM_PAD_INPUT_X_POS, 2+digitCounter*8), ' ', 0, 0xff00, 3);
		digitCounter--;
		break;
	case 13: // abort
		// exit NumPadMode
		numPadActive = 0;
		resetNumString();
		break;
	case 14: //enter
		numPadActive = 0;
		useInput = 1;
		switch (type)
		{
		case ELEMENT_TYPE_INPUT_FLOAT:
		case ELEMENT_TYPE_INPUT_POS_FLOAT:
			sscanf(numString, "%f", (float*)numToBeChangedPtr);
			ConsolePrint("new number: %f\r\n",  *((float*)numToBeChangedPtr)  ); break;
		case ELEMENT_TYPE_INPUT_INT:
			sscanf(numString, "%i", (int32_t*)numToBeChangedPtr);
			ConsolePrint("new number: %i\r\n",  *((int32_t*)numToBeChangedPtr)  ); break;
		case ELEMENT_TYPE_INPUT_POS_INT:
			sscanf(numString, "%u", (uint32_t*)numToBeChangedPtr);
			ConsolePrint("new number: %u\r\n",  *((uint32_t*)numToBeChangedPtr)  );  break;
		}
		resetNumString();
		break;
	}
}


uint16_t GUIDrawButton(uint16_t x1, uint16_t y1, uint8_t * str, uint32_t bGColor, uint32_t fGColor, uint16_t width)
{
	/*
	uint32_t y, x;
	uint16_t xLen;
	xLen= TXT_LEN(str)  7 > width? TXT_LEN(str) 40: width;  //nonsense

	TFTSetPixelArea(x1, y1, xLen, 12, 0);

	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(y = 0; y < 13; y++)
	{
		for(x = 0; x < xLen ; x++)
		{
			if(y == 0 || y == 12 || x == 0 || x == xLen-1)
				TFTWritePixel(fGColor);
			else
				TFTWritePixel(bGColor);
		}
	}
	//GUIDrawString(x1+1, y1+1, str, bGColor, fGColor , 1);
	 * */
	return 2; //xLen;
}// GUIDrawButton
/*
void GUIDrawBox(uint32_t upperLeftX, uint32_t upperLeftY, uint32_t lowerRightX, uint32_t lowerRightY, uint32_t color)
{
	int16_t i;
	TFTSetPixelArea(upperLeftX, upperLeftY, lowerRightX - upperLeftX + 1, 1, 0);
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(i = 0; i < lowerRightX - upperLeftX + 1; i++)
		TFTWritePixel(color);

	TFTSetPixelArea(lowerRightX, upperLeftY + 1, 1, lowerRightY - upperLeftY -1, 0);
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(i = 0; i < lowerRightY - upperLeftY - 1; i++)
		TFTWritePixel(color);

	TFTSetPixelArea(upperLeftX, lowerRightY, lowerRightX - upperLeftX + 1, 1, 0);
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(i = 0; i < lowerRightX - upperLeftX + 1; i++)
		TFTWritePixel(color);

	TFTSetPixelArea(upperLeftX, upperLeftY + 1, 1, lowerRightY - upperLeftY -1, 0);
	*TFT_CMD_REG = TFT_REG_WRITE_MEMORY_START;
	for(i = 0; i < lowerRightY - upperLeftY -1; i++)
		TFTWritePixel(color);
}
*/
