/*
 * presenter.c
 *
 *  Created on: 22.06.2013
 *      Author: AndreR
 */

#include "presenter.h"

#include "intercom1.h"
#include "intercom_constants.h"
#include "config.h"
#include "commands.h"
#include <stdio.h>

Presenter presenter;

static void presenterCb(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	switch(pHeader->command)
	{
		case IC_PRESENTER_CONFIG:
		{
			Config* pCfg = (Config*)pData;

			sprintf(presenter.botIdText, "%hhu", pCfg->botId);

			// TODO: display more

			GUIDrawTabContent();
		}
		break;
		case IC_PRESENTER_KICKER_STATUS:
		{
			KickerStatusV3* pStatus = (KickerStatusV3*)pData;

			sprintf(presenter.kickerChargeText, "%3huV", pStatus->cap/10);

			// TODO: display more
		}
		break;
		case IC_PRESENTER_POWER:
		{
			SystemPowerLog* pLog = (SystemPowerLog*)pData;

			sprintf(presenter.cellText[0], "%humV", pLog->u1);
			sprintf(presenter.cellText[1], "%humV", pLog->u2);
			sprintf(presenter.cellText[2], "%humV", pLog->u3);
			sprintf(presenter.cellText[3], "%humV", pLog->u4);
		}
		break;
		default:
			break;
	}
}

static void dischargeBtn(uint8_t inputNumber)
{
	(void)inputNumber;

	IntercomHeader header;
	header.address = INTERCOM_ADDRESS_MAIN;
	header.section = INTERCOM_SECTION_PRESENTER;
	header.command = IC_PRESENTER_DISCHARGE;

	IntercomSend(&intercom1, &header, 0, 0);
}

void PresenterInit()
{
	sprintf(presenter.botIdText, "%hhu", 42);
	sprintf(presenter.kickerChargeText, "%hu", 4242);

	presenter.tab.pTitle = "CFG";

	GUIElement* pBotIdText = &presenter.elements[0];
	pBotIdText->type = ELEMENT_TYPE_TEXT;
	pBotIdText->fGColor = 0xFF0000;
	pBotIdText->bGColor = 0;
	pBotIdText->pos = PointU16XY(200, 0);
	pBotIdText->text.pText = "BotId:";

	GUIElement* pBotId = &presenter.elements[1];
	pBotId->type = ELEMENT_TYPE_TEXT;
	pBotId->fGColor = 0xFF0000;
	pBotId->bGColor = 0;
	pBotId->pos = PointU16XY(200, 50);
	pBotId->text.pText = presenter.botIdText;

	GUIElement* pKickerChargeText = &presenter.elements[2];
	pKickerChargeText->type = ELEMENT_TYPE_TEXT;
	pKickerChargeText->fGColor = 0x00FF00;
	pKickerChargeText->bGColor = 0;
	pKickerChargeText->pos = PointU16XY(220, 0);
	pKickerChargeText->text.pText = "Kicker:";

	GUIElement* pKickerCharge = &presenter.elements[3];
	pKickerCharge->type = ELEMENT_TYPE_TEXT;
	pKickerCharge->fGColor = 0x00FF00;
	pKickerCharge->bGColor = 0;
	pKickerCharge->pos = PointU16XY(220, 50);
	pKickerCharge->text.pText = presenter.kickerChargeText;

	for(uint8_t i = 0; i < 8; i += 2)
	{
		GUIElement* pPowerText = &presenter.elements[4+i];
		pPowerText->type = ELEMENT_TYPE_TEXT;
		pPowerText->fGColor = 0x00FF00;
		pPowerText->bGColor = 0;
		pPowerText->pos = PointU16XY(80-(i*10), 0);
		pPowerText->text.pText = "Cell:";

		GUIElement* pPower = &presenter.elements[5+i];
		pPower->type = ELEMENT_TYPE_TEXT;
		pPower->fGColor = 0x00FF00;
		pPower->bGColor = 0;
		pPower->pos = PointU16XY(80-(i*10), 50);
		pPower->text.pText = presenter.cellText[i/2];
	}

	GUIElement* pDischargeBtn = &presenter.elements[12];
	pDischargeBtn->bGColor = 0xFF0000;
	pDischargeBtn->fGColor = 0x000000;
	pDischargeBtn->type = ELEMENT_TYPE_BUTTON;
	pDischargeBtn->pos = PointU16XY(100, 100);
	pDischargeBtn->button.size = SizeU16WH(100, 50);
	pDischargeBtn->button.pText = "DISCHARGE";
	pDischargeBtn->button.cb = &dischargeBtn;

	presenter.tab.pElements = presenter.elements;
	presenter.tab.numElements = 13;

	GUIAddTab(&presenter.tab);

	IntercomClient client;
	client.pUserData = 0;
	client.section = INTERCOM_SECTION_PRESENTER;
	client.rxCallback = &presenterCb;

	IntercomAddClient(&intercom1, client);
}
