/*
 * rtc.c
 *
 *  Created on: 30.11.2012
 *      Author: AndreR
 */

#include "rtc.h"

#include "stm32f4xx_rcc.h"
#include "stm32f4xx_rtc.h"
#include "stm32f4xx_pwr.h"

#include <time.h>

void RTCInit()
{
	/* Enable the PWR clock */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR, ENABLE);

	/* Allow access to RTC */
	PWR_BackupAccessCmd(ENABLE);

	/* Reset RTC Domain */
	RCC_BackupResetCmd(ENABLE);
	RCC_BackupResetCmd(DISABLE);

	// Select high-speed external oscillator (divided by 12) as clk source => 1MHz
	RCC_RTCCLKConfig(RCC_RTCCLKSource_HSE_Div12 );

	// Enable RTC Clock
	RCC_RTCCLKCmd(ENABLE);

	// Wait for RTC registers synchronization
	RTC_WaitForSynchro();

	/* Configure the RTC data register and RTC prescaler */
	RTC_InitTypeDef rtcInit;
	rtcInit.RTC_AsynchPrediv = 125;
	rtcInit.RTC_SynchPrediv	 = 8000;
	rtcInit.RTC_HourFormat   = RTC_HourFormat_24;
	RTC_Init(&rtcInit);

	/* Set the time to 00h 00mn 00s AM */
	RTC_TimeTypeDef rtcTime;
	rtcTime.RTC_H12     = RTC_H12_AM;
	rtcTime.RTC_Hours   = 0x00;
	rtcTime.RTC_Minutes = 0x00;
	rtcTime.RTC_Seconds = 0x00;
	RTC_SetTime(RTC_Format_BIN, &rtcTime);

	RTC_DateTypeDef rtcDate;
	rtcDate.RTC_WeekDay = RTC_Weekday_Tuesday;
	rtcDate.RTC_Date = 1;
	rtcDate.RTC_Month = 1;
	rtcDate.RTC_Year = 13;
	RTC_SetDate(RTC_Format_BIN, &rtcDate); // init to 1.1.2013
}

// Return unix timestamp
uint32_t RTCGetTime()
{
	RTC_TimeTypeDef rtcTime;
	RTC_GetTime(RTC_Format_BIN, &rtcTime);

	RTC_DateTypeDef rtcDate;
	RTC_GetDate(RTC_Format_BIN, &rtcDate);

	struct tm t;
	t.tm_year = rtcDate.RTC_Year + 100;
	t.tm_mon = rtcDate.RTC_Month-1; // Month, 0 - jan
	t.tm_mday = rtcDate.RTC_Date; // Day of the month
	t.tm_hour = rtcTime.RTC_Hours;
	t.tm_min = rtcTime.RTC_Minutes;
	t.tm_sec = rtcTime.RTC_Seconds;
	t.tm_isdst = -1; // Is DST on? 1 = yes, 0 = no, -1 = unknown

	time_t time;
	time = mktime(&t);

	return (uint32_t)time;
}

void RTCSetTime(uint32_t unixTime)
{
	time_t time = (time_t)unixTime;

	struct tm* pT = localtime(&time);

	RTC_TimeTypeDef rtcTime;
	rtcTime.RTC_H12     = RTC_H12_AM;
	rtcTime.RTC_Hours   = pT->tm_hour;
	rtcTime.RTC_Minutes = pT->tm_min;
	rtcTime.RTC_Seconds = pT->tm_sec;
	RTC_SetTime(RTC_Format_BIN, &rtcTime);

	RTC_DateTypeDef rtcDate;
	rtcDate.RTC_WeekDay = ((pT->tm_wday+6)%7)+1;
	rtcDate.RTC_Date = pT->tm_mday;
	rtcDate.RTC_Month = pT->tm_mon+1;
	rtcDate.RTC_Year = pT->tm_year-100;
	RTC_SetDate(RTC_Format_BIN, &rtcDate);
}

DateTime RTCGetDateTime()
{
	DateTime result;

	RTC_TimeTypeDef rtcTime;
	RTC_GetTime(RTC_Format_BIN, &rtcTime);

	RTC_DateTypeDef rtcDate;
	RTC_GetDate(RTC_Format_BIN, &rtcDate);

	result.year = rtcDate.RTC_Year+2000;
	result.month = rtcDate.RTC_Month;
	result.day = rtcDate.RTC_Date;
	result.hour = rtcTime.RTC_Hours;
	result.min = rtcTime.RTC_Minutes;
	result.sec = rtcTime.RTC_Seconds;

	return result;
}
