/*
 * init.c
 *
 *  Created on: 17.05.2011
 *      Author: AndreR
 */

#include "init.h"

#include "stm32f4xx_iwdg.h"

#include "FreeRTOS.h"
#include "task.h"

#include "hal/led.h"
#include "hal/usart2.h"
#include "hal/exti.h"
#include "hal/rtc.h"
#include "hal/tft.h"
#include "hal/hw_sync1.h"
#include "hal/fsmc.h"
#include "hal/sram.h"
#include "hal/ov7725.h"

#include "intercom1.h"
#include "intercom_constants.h"
#include "cli.h"
#include "spi3.h"
#include "display.h"
#include "gui.h"
#include "audio.h"
#include "usb.h"
#include "sensor_log.h"
#include "bootloader.h"
#include "gui_demo.h"
#include "presenter.h"

void InitTask(void* params)
{
	params = params;

	// -- HAL
	RTCInit();
	FSMCInit();
	SRAMInit();
	ExtiInit();
	USART2Init();
	TFTInit();
	OV7725Init();

	// Gate Keepers
	SPI3Init();

	// -- High-Level systems
	Intercom1Init();
	CLIInit();
	DisplayInit();
	GUIInit();
	USBInit();
	SensorLogInit();
	BootloaderInit();
	PresenterInit();

	HWSyncBarrier(&hwSync1);

//	GUIDemoInit();

	// Create Tasks, MAX PRIORITY: 4
	xTaskCreate(Intercom1Task, (const signed char* const)"IC", 1024, NULL, 1, NULL);
	xTaskCreate(CLITask, (const signed char* const)"CLI", 2048, NULL, 1, NULL);
	xTaskCreate(SPI3Task, (const signed char* const)"SPI3", 256, NULL, 2, NULL);
	xTaskCreate(DisplayTask, (const signed char* const)"Display", 256, NULL, 1, NULL);
	xTaskCreate(AudioTask, (const signed char* const)"Audio", 256, NULL, 1, NULL);
	xTaskCreate(HCDTask, (const signed char* const)"USB HCD", 512, &USB_OTG_Core, 4, NULL);
	xTaskCreate(USBTask, (const signed char* const)"USB", 1024, NULL, 3, NULL);
	xTaskCreate(USARTTask, (const signed char* const)"USART2", 1024, &usart2, 4, NULL);
	vTaskDelay(100/portTICK_RATE_MS);	// give USART some time to reset RX

	ConsolePrint("%d Tasks started\r\n", uxTaskGetNumberOfTasks());
	ConsolePrint("HEAP usage: %d/%d Byte\r\n", configTOTAL_HEAP_SIZE-xPortGetFreeHeapSize(), configTOTAL_HEAP_SIZE);

	vTaskSuspend(NULL);	// Disable this task
}
