/*
 * sensor_log.c
 *
 *  Created on: 02.04.2013
 *      Author: AndreR
 */

#include "sensor_log.h"

#include "intercom1.h"
#include "util/console.h"
#include "intercom_constants.h"

#include <stdio.h>

SensorLog sensorLog;

static void sensorCb(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	switch(pHeader->command)
	{
		case 0:	// data
		{
			float* pValues = (float*)pData;

			int32_t length = sprintf(sensorLog.buf+sensorLog.bytesUsed, "%.5f, %.5f, %.5f\r\n", pValues[0], pValues[1], pValues[2]);
			sensorLog.bytesUsed += length;

			if(SENSOR_LOG_BUF_SIZE-sensorLog.bytesUsed < 100)
			{
				uint32_t written = 0;
//				ConsolePrint("write %uB\r\n", sensorLog.bytesUsed);
				if(f_write(&sensorLog.logFile, sensorLog.buf, sensorLog.bytesUsed, (UINT*)&written) != FR_OK)
				{
//					ConsolePrint("Write failed\r\n");
				}
				sensorLog.bytesUsed = 0;
			}
		}
		break;
		case 1: // start
		{
			char filename[16];
			sprintf(filename, "move/%03u.csv", (UINT)sensorLog.logCounter);

			if(f_open(&sensorLog.logFile, filename, FA_WRITE | FA_CREATE_ALWAYS) != FR_OK)
			{
				ConsolePrint("open move log file failed\r\n");
			}

			sensorLog.logCounter++;
			sensorLog.bytesUsed = 0;
		}
		break;
		case 2: // stop
		{
			uint32_t written = 0;
			f_write(&sensorLog.logFile, sensorLog.buf, sensorLog.bytesUsed, (UINT*)&written);

			f_close(&sensorLog.logFile);
		}
		break;
	}
}

void SensorLogInit()
{
	IntercomClient client;
	client.pUserData = 0;
	client.section = INTERCOM_SECTION_SENSORS;
	client.rxCallback = &sensorCb;

	IntercomAddClient(&intercom1, client);

	sensorLog.logCounter = 0;
	sensorLog.bytesUsed = 0;
}
